/**
 * @file
 * Library manager behaviors.
 */

(function ($, Drupal) {

  'use strict';

  /**
   * Initialize CodeMirror library.
   */
  Drupal.behaviors.editor = {
    attach: function (context, settings) {

      /* global CodeMirror */
      if (typeof CodeMirror == 'undefined') {
        alert(Drupal.t('CodeMirror library is not loaded!'));
        return;
      }

      var $editorTextArea = $('textarea[data-drupal-selector="edit-code"]');
      if ($editorTextArea[0]) {
        CodeMirror.fromTextArea($editorTextArea[0], {
          lineNumbers: true,
          mode: $editorTextArea.attr('data-editor-mode'),
          extraKeys: {
            F11: function (editor) {
              editor.setOption('fullScreen', !editor.getOption('fullScreen'));
            },
            Esc: function (editor) {
              editor.setOption('fullScreen', false);
            }
          }
        });
        $editorTextArea
          .closest('.form-textarea-wrapper')
          .addClass('codemirror-textarea-wrapper');
      }

      var $exportTextArea = $('textarea[data-drupal-selector="library-export"]');
      if ($exportTextArea[0]) {
          CodeMirror.fromTextArea($exportTextArea[0], {
          mode: 'yaml',
          viewportMargin: 1000}
        );
        $exportTextArea
          .closest('.form-textarea-wrapper')
          .addClass('codemirror-textarea-wrapper')
          .addClass('library-export-wrapper');
      }

    }
  };

  /**
   * Filters the library list table by a text input search string.
   */
  Drupal.behaviors.libraryFilterByText = {
    attach: function (context, settings) {

      var $input = $('[data-drupal-selector="library-filter"]');
      var $table = $('[data-drupal-selector="library-list"]');
      var $rows = $table.find('tbody tr');

      $table.find('tbody').append('<tr class="empty-row"/>');
      var $emptyRow = $('.empty-row');
      $emptyRow
        .hide()
        .append('<td colspan="5">' + Drupal.t('No libraries were found.') + '</td>');

      function filterLibraryList(event) {
        var query = $(event.target).val();
        var regExp = new RegExp(query, 'i');

        if (query.length >= 0) {
          $rows.each(function (index, row) {
            var $rows = $(row);
            var text = $rows.find('td:eq(0)').text();
            $rows.toggle(text.search(regExp) !== -1);
          });
        }

        $emptyRow.toggle($rows.filter(':visible').length === 0);
      }

      $input.on({keyup: Drupal.debounce(filterLibraryList, 100)});
    }
  };

}(jQuery, Drupal));
