<?php

namespace Drupal\library_manager\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Library manager settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'library_manager_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['library_manager.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $settings = $this->config('library_manager.settings')->get();

    $form['codemirror'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    $form['codemirror']['cdn'] = [
      '#type' => 'checkbox',
      '#title' => t('Load CodeMirror library from CDN'),
      '#default_value' => $settings['codemirror']['cdn'],
    ];

    $form['libraries_path'] = [
      '#type' => 'textfield',
      '#title' => t('Libraries path'),
      '#description' => t('A local file system path where library files will be stored. This directory should not be writrable by web server and accessible over the web.'),
      '#after_build' => array('system_check_directory'),
      '#default_value' => $settings['libraries_path'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('library_manager.settings')
      ->set('codemirror', $form_state->getValue('codemirror'))
      ->set('libraries_path', $form_state->getValue('libraries_path'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
