<?php

namespace Drupal\library_manager\Tests;

/**
 * Tests the Library definition interface.
 *
 * @group library_manager
 */
class LibraryDefinitionTest extends TestBase {

  /**
   * Tests collection page.
   */
  public function testDefinitionCollectionPage() {
    $this->drupalGet('admin/structure/library/definition');

    $this->assertPageTitle('Library definitions');

    $expected_header = [
      'Machine name',
      'Version',
      'License',
      'Operations',
    ];
    $this->assertTableHeader('//main//table//th', $expected_header);

    $alpha_row = $this->xpath('//main//table//td[position() = 1 and text() = "alpha"]/../td');
    $this->assertEqual($alpha_row[1], '1.2.3');
    $this->assertEqual($alpha_row[2], 'GNU-GPL-2.0-or-later');
    $edit_link = $alpha_row[3]->xpath('//a[text() = "Edit"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($edit_link['href']));
    $this->assertPageTitle('Edit library');

    $this->drupalPostForm(NULL, ['version' => '1.2.4'], 'Save');
    $this->assertStatusMessage('Library definition has been updated.');
    $this->assertByXpath('//main//table//td[position() = 1 and text() = "alpha"]/following-sibling::td[position() = 1 and text() = "1.2.4"]');

    $edit_link = $alpha_row[3]->xpath('//a[text() = "Delete"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($edit_link['href']));
    $this->drupalPostForm(NULL, [], 'Delete');
    $this->assertText('There is no Library definition yet.');
  }

  /**
   * Tests definition form.
   */
  public function testDefinitionForm() {
    $this->drupalGet('admin/structure/library/definition/add');
    $this->assertPageTitle('Add library definition');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-id")]/label[text() = "Machine-readable name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-version")]/label[text() = "Version"]');
    $this->assertByXpath('//div[contains(@class, "form-item-license")]/label[text() = "License"]');

    $header = ['Name', 'Size', 'Type', 'Operations'];

    $this->assertByXpath('//fieldset[@id="edit-js"]/legend/span[text() = "JS files"]');
    $this->assertTableHeader('//fieldset[@id="edit-js"]//th', $header);
    $this->assertByXpath('//fieldset[@id="edit-js"]//table//td[text() = "JS files are not configured yet."]');

    $this->assertByXpath('//fieldset[@id="edit-css"]/legend/span[text() = "CSS files"]');
    $this->assertTableHeader('//fieldset[@id="edit-css"]//th', $header);
    $this->assertByXpath('//fieldset[@id="edit-css"]//table//td[text() = "CSS files are not configured yet."]');

    $this->assertByXpath('//fieldset[@id="edit-dependencies"]/legend/span[text() = "Dependencies"]');
    $this->assertByXpath('//fieldset[@id="edit-dependencies"]//input[@type="submit" and @value="Add dependency"]');

    $this->assertByXpath('//div[contains(@class, "form-item-load")]/label[text() = "Load the library automatically according to visibility rules"]');

    $tabs_prefix = '//div[contains(@class, "form-type-vertical-tabs")]';
    $this->assertByXpath($tabs_prefix . '/label[text() = "Visibility"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/summary[text() = "Current Theme"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/div//label[text() = "Theme"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-current-theme"]/div//label[text() = "Negate the condition"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/summary[text() = "Request Path"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/div//label[text() = "Pages"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-request-path"]/div//label[text() = "Negate the condition"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/summary[text() = "User Role"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/div//fieldset/legend/span[text() = "When the user has the following roles"]');
    $this->assertByXpath($tabs_prefix . '//details[@id="edit-visibility-user-role"]/div//label[text() = "Negate the condition"]');

    $edit = [
      'id' => 'foo',
      'version' => '3.4.5',
      'license' => 'MIT',
      'library_dependencies[0]' => 'core/jquery',
      'load' => TRUE,
      'visibility[current_theme][theme]' => 'classy',
      'visibility[current_theme][negate]' => TRUE,
      'visibility[request_path][pages]' => '/user',
      'visibility[user_role][roles][anonymous]' => TRUE,
    ];

    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertStatusMessage('Library definition has been updated.');

    // Check default form values for newly created definition.
    $this->drupalGet('admin/structure/library/definition/foo/edit');

    $this->assertByXpath('//input[@name="id" and @value="foo"]');
    $this->assertByXpath('//input[@name="version" and @value="3.4.5"]');
    $this->assertByXpath('//input[@name="license" and @value="MIT"]');
    $this->assertByXpath('//input[@name="library_dependencies[0]" and @value="core/jquery"]');
    $this->assertByXpath('//input[@name="load" and @checked="checked"]');
    $this->assertByXpath('//select[@name="visibility[current_theme][theme]"]/option[@value = "classy" and @selected = "selected"]');
    $this->assertByXpath('//input[@name="visibility[current_theme][negate]" and @checked = "checked"]');
    $this->assertByXpath('//textarea[@name="visibility[request_path][pages]" and text() = "/user"]');
    $this->assertByXpath('//input[@name="visibility[request_path][negate]" and not(@checked)]');
    $this->assertByXpath('//input[@name="visibility[user_role][roles][anonymous]" and @checked = "checked"]');
    $this->assertByXpath('//input[@name="visibility[user_role][negate]" and not(@checked)]');

    // Check delete form.
    $delete_url = $this->getAbsoluteUrl($this->xpath('//a[@id="edit-delete"]')[0]['href']);
    $this->drupalGet($delete_url);
    $this->assertPageTitle(t('Are you sure you want to delete the library definition %definition?', ['%definition' => 'foo']));
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Library definitions');
    $this->drupalGet($delete_url);
    $this->drupalPostForm(NULL, [], 'Delete');
    $this->assertStatusMessage(t('Deleted library definition %definition_name.', ['%definition_name' => 'foo']));
    $this->assertPageTitle('Library definitions');
  }

  /**
   * Tests JS file form.
   */
  public function testJsFileForm() {
    $this->drupalGet('admin/structure/library/definition/alpha/edit');
    $this->clickLink('Add JS file');

    $this->assertPageTitle('Add JS file');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-file-name")]/label[text() = "File name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-preprocess")]/label[text() = "Preprocess"]');
    $this->assertByXpath('//div[contains(@class, "form-item-minified")]/label[text() = "Minified"]');
    $this->assertByXpath('//div[contains(@class, "form-item-external")]/label[text() = "External"]');
    $this->assertByXpath('//div[contains(@class, "form-item-code")]/label[text() = "Code"]');

    $edit = [
      'file_name' => 'foo.js',
      'preprocess' => FALSE,
      'minified' => TRUE,
      'external' => FALSE,
      'code' => 'console.log(123);',
    ];

    $this->drupalPostForm(NULL, $edit, 'Save');

    $this->assertStatusMessage('The JS file has been saved.');
    $this->assertPageTitle('Edit library');

    $edit_js_link = $this->xpath('//fieldset[@id="edit-js"]//td[text() = "foo.js"]/following-sibling::td[position() = 3]//a[text() = "Edit"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($edit_js_link['href']));

    $this->assertPageTitle('Edit JS file');

    // Check default form values for newly created JS file.
    $this->assertByXpath('//input[@name="file_name" and @value="foo.js"]');
    $this->assertByXpath('//input[@name="preprocess" and not(@checked)]');
    $this->assertByXpath('//input[@name="minified" and @checked = "checked"]');
    $this->assertByXpath('//input[@name="external" and not(@checked)]');
    $this->assertByXpath('//textarea[@name="code" and text() = "console.log(123);"]');

    $this->clickLink('Delete');

    $this->assertPageTitle('Are you sure you want to delete the file?');
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Edit library');

    $delete_js_link = $this->xpath('//fieldset[@id="edit-js"]//td[text() = "foo.js"]/following-sibling::td[position() = 3]//a[text() = "Delete"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($delete_js_link['href']));

    $this->drupalPostForm(NULL, [], 'Delete');

    $this->assertStatusMessage('The JS file has been deleted.');
    $this->assertPageTitle('Edit library');
  }

  /**
   * Tests CSS file form.
   */
  public function testCssFileForm() {
    $this->drupalGet('admin/structure/library/definition/alpha/edit');
    $this->clickLink('Add CSS file');

    $this->assertPageTitle('Add CSS file');

    // Check form labels.
    $this->assertByXpath('//div[contains(@class, "form-item-file-name")]/label[text() = "File name"]');
    $this->assertByXpath('//div[contains(@class, "form-item-preprocess")]/label[text() = "Preprocess"]');
    $this->assertByXpath('//div[contains(@class, "form-item-minified")]/label[text() = "Minified"]');
    $this->assertByXpath('//div[contains(@class, "form-item-external")]/label[text() = "External"]');
    $this->assertByXpath('//div[contains(@class, "form-item-url")]/label[text() = "Url"]');
    $this->assertByXpath('//div[contains(@class, "form-item-code")]/label[text() = "Code"]');

    $edit = [
      'file_name' => 'foo.css',
      'preprocess' => FALSE,
      'minified' => TRUE,
      'external' => FALSE,
      'url' => 'http://example.com/foo.css',
    ];

    $this->drupalPostForm(NULL, $edit, 'Save');

    $this->assertStatusMessage('The CSS file has been saved.');
    $this->assertPageTitle('Edit library');

    $edit_js_link = $this->xpath('//fieldset[@id="edit-css"]//td[text() = "foo.css"]/following-sibling::td[position() = 3]//a[text() = "Edit"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($edit_js_link['href']));

    $this->assertPageTitle('Edit CSS file');

    // Check default form values for newly created JS file.
    $this->assertByXpath('//input[@name="file_name" and @value="foo.css"]');
    $this->assertByXpath('//input[@name="preprocess" and not(@checked)]');
    $this->assertByXpath('//input[@name="minified" and @checked = "checked"]');
    $this->assertByXpath('//input[@name="external" and not(@checked)]');
    $this->assertByXpath('//input[@name="url" and @value="http://example.com/foo.css"]');

    $this->clickLink('Delete');

    $this->assertPageTitle('Are you sure you want to delete the file?');
    $this->assertText('This action cannot be undone.');
    $this->clickLink('Cancel');
    $this->assertPageTitle('Edit library');

    $delete_js_link = $this->xpath('//fieldset[@id="edit-css"]//td[text() = "foo.css"]/following-sibling::td[position() = 3]//a[text() = "Delete"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($delete_js_link['href']));

    $this->drupalPostForm(NULL, [], 'Delete');

    $this->assertStatusMessage('The CSS file has been deleted.');
    $this->assertPageTitle('Edit library');
  }

}
